#ifndef CHARSET_H
#define CHARSET_H

void init_charset(void);

/* 
 * We'd like char8 to be unsigned char, but it causes too many problems.
 * For example:
 *     char8 name; ...; return name ? name : "<none>"
 * produces a warning with many compilers if char8 is unsigned.
 */

typedef char char8;
typedef unsigned short char16;
typedef unsigned int char32;

#if CHAR_SIZE == 8
typedef char8 Char;
#elif CHAR_SIZE == 16
typedef char16 Char;
#else
#error CHAR_SIZE must be 8 or 16
#endif

/* Character encodings */

enum character_encoding {
    CE_unknown,
    CE_UTF_8, CE_ISO_8859_1,
    CE_UTF_16B, CE_UTF_16L, CE_ISO_10646_UCS_2B, CE_ISO_10646_UCS_2L, 
    CE_enum_count
};

typedef enum character_encoding CharacterEncoding;

extern CharacterEncoding InternalCharacterEncoding;

extern const char8 *CharacterEncodingName[CE_enum_count];
extern const char8 *CharacterEncodingNameAndByteOrder[CE_enum_count];

struct character_encoding_alias {const char8 *name; CharacterEncoding enc;};
extern struct character_encoding_alias CharacterEncodingAlias[];
extern const int CE_alias_count;

int EncodingsCompatible(CharacterEncoding enc1, CharacterEncoding enc2,
			CharacterEncoding *enc3);

#endif /* CHARSET_H */
