/* 	$Id: xmlparser.h,v 1.11 1998/02/13 15:12:09 richard Exp $    */

/* Typedefs */

typedef struct parser_state *Parser;
typedef struct attribute *Attribute;
typedef struct xbit *XBit;
typedef void CallbackProc(XBit bit, void *arg);

/* Bits */

void FreeXBit(XBit xbit);

enum xbit_type {
    XBIT_dtd,
    XBIT_start, XBIT_empty, XBIT_end, XBIT_eof, XBIT_pcdata,
    XBIT_pi, XBIT_comment, XBIT_cdsect, XBIT_xml, 
    XBIT_error, XBIT_warning, XBIT_none,
    XBIT_enum_count
};
typedef enum xbit_type XBitType;

extern const char8 *XBitTypeName[XBIT_enum_count];

struct attribute {
    AttributeDefinition definition; /* The definition of this attribute */
    Char *value;		/* The (possibly normalised) value */
    int quoted;			/* Was it quoted? */
    struct attribute *next;	/* The next attribute or null */
};

struct xbit {
    Entity entity;
    int byte_offset;
    enum xbit_type type;
    char8 *s1, *s2;
    Char *S1, *S2;
    int i1, i2;
    Attribute attributes;
    ElementDefinition element_definition;
    Entity e1, e2;
};

#define dtd_name S1
#define dtd_internal_part e1
#define dtd_external_part e2

#define pcdata_chars S1

#define pi_name S1
#define pi_chars S2

#define comment_chars S1

#define cdsect_chars S1

#define xml_version s1
#define xml_encoding_name s2
#define xml_standalone i1
#define xml_encoding i2

#define error_message s1

/* Parser flags */

enum parser_flag {
    ExpandCharacterEntities,
    ExpandGeneralEntities,
    XMLPiEnd,
    XMLEmptyTagEnd,
    XMLPredefinedEntities,
    WarnOnUnquotedAttributeValues,
    NormaliseAttributeValues,
    NormalizeAttributeValues,
    ErrorOnBadCharacterEntities,
    ErrorOnUndefinedEntities,
    ReturnComments,
    CaseInsensitive,
    ErrorOnUndefinedElements,
    WarnOnUndefinedElements,
    ErrorOnUndefinedAttributes,
    WarnOnUndefinedAttributes,
    WarnOnRedefinitions,
    TrustSDD,
    XMLExternalIDs,
    ReturnDefaultedAttributes,
    MergePCData,
    XMLMiscWFWarnings,
    ErrorOnXMLWFErrors,
    AllowMultipleElements,
    CheckEndTagsMatch
};
typedef enum parser_flag ParserFlag;

/* Parser */

enum standalone_declaration {
    /* NB must match NSL's rmdCode */
    SDD_unspecified, SDD_no, SDD_yes, SDD_enum_count
};
typedef enum standalone_declaration StandaloneDeclaration;

extern const char8 *StandaloneDeclarationName[SDD_enum_count];

enum parse_state {PS_prolog, PS_body, PS_epilog, PS_end, PS_error};

struct element_info {
    ElementDefinition definition;
    Entity entity;
};

struct parser_state {
    enum parse_state state;
    Entity document_entity;
    int have_dtd;		/* True if dtd has been processed */
    StandaloneDeclaration standalone;
    struct input_source *source;
    Char *name, *pbuf;
    int namelen, pbufsize, pbufnext;
    struct xbit xbit;
    int peeked;
    Dtd dtd;			/* The document's DTD */
    CallbackProc *dtd_callback;
    CallbackProc *warning_callback;
    unsigned int flags;
    struct element_info *element_stack;
    int element_stack_alloc;
    int element_depth;
    void *callback_arg;
    int external_pe_depth;	/* To keep track of whether we're in the */
				/* internal subset: 0 <=> yes */
};

int ParserInit(void);
Parser NewParser(void);
void ParserClose(Parser p);

Entity ParserRootEntity(Parser p);
InputSource ParserRootSource(Parser p);

XBit ReadXBit(Parser p);
XBit PeekXBit(Parser p);

XBit ParseDtd(Parser p, Entity e);

void ParserSetWarningCallback(Parser p, CallbackProc cb);
void ParserSetDtdCallback(Parser p, CallbackProc cb);
void ParserSetCallbackArg(Parser p, void *arg);

int ParserPush(Parser p, InputSource source);
void ParserPop(Parser p);

void ParserSetFlag(Parser p,  ParserFlag flag, int value);
#define ParserGetFlag(p, flag) ((p)->flags & (1 << (flag)))

void ParserPerror(Parser p, XBit bit);

